import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:puzzlegame/config/default_game_settings.dart';
import 'package:puzzlegame/models/settings/settings_game.dart';

part 'settings_game_state.dart';

class GameSettingsCubit extends HydratedCubit<GameSettingsState> {
  GameSettingsCubit() : super(GameSettingsState(settings: GameSettings.createDefault()));

  void setValues({
    String? tilesetSize,
    String? imageName,
  }) {
    emit(
      GameSettingsState(
        settings: GameSettings(
          tilesetSize: tilesetSize ?? state.settings.tilesetSize,
          imageName: imageName ?? state.settings.imageName,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultGameSettings.parameterCodeTilesetSize:
        return GameSettings.getTilesetSizeValueFromUnsafe(state.settings.tilesetSize);
      case DefaultGameSettings.parameterCodeImageName:
        return GameSettings.getImageNameFromUnsafe(state.settings.imageName);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String tilesetSize = code == DefaultGameSettings.parameterCodeTilesetSize
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeTilesetSize);
    final String imageName = code == DefaultGameSettings.parameterCodeImageName
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeImageName);

    setValues(
      tilesetSize: tilesetSize,
      imageName: imageName,
    );
  }

  @override
  GameSettingsState? fromJson(Map<String, dynamic> json) {
    final String tilesetSize = json[DefaultGameSettings.parameterCodeTilesetSize] as String;
    final String imageName = json[DefaultGameSettings.parameterCodeImageName] as String;

    return GameSettingsState(
      settings: GameSettings(
        tilesetSize: tilesetSize,
        imageName: imageName,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(GameSettingsState state) {
    return <String, dynamic>{
      DefaultGameSettings.parameterCodeTilesetSize: state.settings.tilesetSize,
      DefaultGameSettings.parameterCodeImageName: state.settings.imageName,
    };
  }
}
