import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:puzzlegame/data/fetch_data_helper.dart';
import 'package:puzzlegame/models/game/moving_tile.dart';
import 'package:puzzlegame/models/settings/settings_game.dart';
import 'package:puzzlegame/models/settings/settings_global.dart';

class Game {
  Game({
    // Settings
    required this.gameSettings,
    required this.globalSettings,

    // State
    this.isRunning = false,
    this.isStarted = false,
    this.isFinished = false,
    this.animationInProgress = false,
    this.shufflingInProgress = false,

    // Base data
    required this.image,
    this.tiles = const [],

    // Game data
    this.movesCount = 0,
    this.displayTip = false,
    this.tileSize = 0.0,
  });

  // Settings
  final GameSettings gameSettings;
  final GlobalSettings globalSettings;

  // State
  bool isRunning;
  bool isStarted;
  bool isFinished;
  bool animationInProgress;
  bool shufflingInProgress;

  // Base data
  String image;
  List<MovingTile> tiles;

  // Game data
  int movesCount;
  bool displayTip;
  double tileSize;

  factory Game.createEmpty() {
    return Game(
      gameSettings: GameSettings.createDefault(),
      globalSettings: GlobalSettings.createDefault(),
      image: '',
      tiles: [],
    );
  }

  factory Game.createNew({
    GameSettings? gameSettings,
    GlobalSettings? globalSettings,
  }) {
    final GameSettings newGameSettings = gameSettings ?? GameSettings.createDefault();
    final GlobalSettings newGlobalSettings = globalSettings ?? GlobalSettings.createDefault();

    final String image = FetchDataHelper().getRandomItem();

    return Game(
      // Settings
      gameSettings: newGameSettings,
      globalSettings: newGlobalSettings,
      // Base data
      image: image,
    );
  }

  bool get canBeResumed => isStarted && !isFinished;

  void dump() {
    printlog('');
    printlog('## Current game dump:');
    printlog('');
    printlog('$Game:');
    printlog('  Settings');
    gameSettings.dump();
    globalSettings.dump();
    printlog('  State');
    printlog('    isRunning: $isRunning');
    printlog('    isStarted: $isStarted');
    printlog('    isFinished: $isFinished');
    printlog('    animationInProgress: $animationInProgress');
    printlog('    shufflingInProgress: $shufflingInProgress');
    printlog('  Base data');
    printlog('    image: $image');
    printlog('  Game data');
    printlog('    movesCount: $movesCount');
    printlog('    displayTip: $displayTip');
    printlog('    tileSize: $tileSize');
    printlog('');
  }

  @override
  String toString() {
    return '$Game(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      // Settings
      'gameSettings': gameSettings.toJson(),
      'globalSettings': globalSettings.toJson(),
      // State
      'isRunning': isRunning,
      'isStarted': isStarted,
      'isFinished': isFinished,
      'animationInProgress': animationInProgress,
      'shufflingInProgress': shufflingInProgress,
      // Base data
      'image': image,
      'tiles': tiles,
      // Game data
      'movesCount': movesCount,
      'displayTip': displayTip,
      'tileSize': tileSize,
    };
  }
}
